package com.sd.wln.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.wln.service.WlnOrderService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WlnOrderServiceImpl implements WlnOrderService {

    @Autowired
    private PlatformTransactionManager platformTransactionManager;

    @Autowired
    private TransactionDefinition transactionDefinition;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Override
    public boolean syncOrder() {
        Date endDate = new Date();

        List<JSONObject> wlnOrderList = getWlnOrderList(endDate);
        if (wlnOrderList == null) {
            log.error("订单同步失败");
            return false;
        }

        List<OrderInfo> saveOrderListInfo = new ArrayList<>();
        List<OrderSku> saveOrderSkuList = new ArrayList<>();
        List<OrderInfo> updateOrderListInfo = new ArrayList<>();

        Map<String, Department> departmentMap = getDepartmentMap();

        Map<String, OrderInfo> orderMap = orderService.mapKEntity(OrderInfo::getWlnUid,
                q -> q.between(OrderInfo::getWlnApproveTime, DateUtil.offsetDay(endDate, -3), endDate));

        Map<String, SkuSpec> skuSpecMap = getSkuSpecMap(wlnOrderList);

        for (JSONObject wlnOrder : wlnOrderList) {

            // 如果订单存在,修改订单状态
            OrderInfo orderInfo = orderMap.get(wlnOrder.getString("uid"));
            if (orderInfo != null) {
                Integer processStatus = wlnOrder.getInteger("process_status");
                if (ObjectUtil.notEqual(orderInfo.getWlnStatus(), processStatus)) {
                    orderInfo.setWlnStatus(processStatus);
                    updateOrderListInfo.add(orderInfo);
                }
                continue;
            }

            orderInfo = createOrder(wlnOrder, departmentMap);
            for (JSONObject wlnOrderSku : wlnOrder.getJSONArray("orders").toJavaList(JSONObject.class)) {
                SkuSpec skuSpec = skuSpecMap.get(wlnOrderSku.getString("sys_spec_uid"));
                OrderSku orderSku = createOrderSku(wlnOrderSku, orderInfo, skuSpec);
                saveOrderSkuList.add(orderSku);
            }
            saveOrderListInfo.add(orderInfo);
        }

        // 开启事务
        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
        try {
            if (saveOrderListInfo.size() > 0) {
                orderService.saveBatch(saveOrderListInfo);
            }
            if (updateOrderListInfo.size() > 0) {
                orderService.updateBatchById(updateOrderListInfo);
            }
            if (saveOrderSkuList.size() > 0) {
                orderSkuService.saveBatch(saveOrderSkuList);
            }
            // 提交事务
            platformTransactionManager.commit(transactionStatus);
            return true;
        } catch (Exception e) {
            // 回滚事务
            platformTransactionManager.rollback(transactionStatus);
            log.error("同步订单失败", e);
            return false;
        }
    }

    /**
     * 万里牛查询3天之内的全部订单
     */
    private List<JSONObject> getWlnOrderList(Date endDate) {
        List<DictCommonData> warehouseCodeList = dictCommonDataService.list(q -> q
                .eq(DictCommonData::getDictCode, "warehouse_code"));
        if (warehouseCodeList.size() == 0) {
            log.error("仓库字典:warehouse_code 为空");
            return null;
        }

        List<JSONObject> list = new ArrayList<>();

        long endTime = endDate.getTime();
        long startTime = endTime - 1000 * 60 * 60 * 24 - 1;

        for (int i = 0; i < 3; i++) {
            for (DictCommonData dictCommonData : warehouseCodeList) {
                int page = 1;
                int size;
                do {
                    try {
                        List<JSONObject> itemList = WlnUtil.getOrderList(page, 200, startTime, endTime, dictCommonData.getDictValue());
                        page++;
                        size = itemList.size();
                        list.addAll(itemList);
                    } catch (Exception e) {
                        log.error("订单同步失败", e);
                        return null;
                    }
                } while (size >= 200);
            }
            startTime -= 1000 * 60 * 60 * 24;
            endTime -= 1000 * 60 * 60 * 24;
        }

        return list;
    }

    /**
     * 事业部id Map
     */
    private Map<String, Department> getDepartmentMap() {
        Map<String, Department> departmentMap = new HashMap<>();
        for (Department department : departmentService.list()) {
            String warehouseCode = department.getWlnWarehouseCode();
            if (StrUtil.isBlank(warehouseCode)) {
                continue;
            }
            for (String item : warehouseCode.split(",")) {
                departmentMap.put(item, department);
            }
        }
        return departmentMap;
    }

    /**
     * 获取sku规格
     */
    private Map<String, SkuSpec> getSkuSpecMap(List<JSONObject> wlnOrderList) {
        List<String> wlnUidList = wlnOrderList.stream()
                .flatMap(item -> item.getJSONArray("orders").toJavaList(JSONObject.class).stream())
                .map(item -> item.getString("sys_spec_uid"))
                .filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());
        return skuSpecService.mapKEntity(SkuSpec::getWlnUid, q -> q.in(SkuSpec::getWlnUid, wlnUidList));
    }

    /**
     * 创建订单
     */
    private OrderInfo createOrder(JSONObject wlnOrder, Map<String, Department> departmentMap) {
        Department department = departmentMap.get(wlnOrder.getString("storage_code"));

        OrderInfo orderInfo = new OrderInfo();
        orderInfo.setId(IdWorker.getId());
        orderInfo.setSource(2);
        orderInfo.setCode(wlnOrder.getString("tp_tid"));
        orderInfo.setWlnCode(wlnOrder.getString("trade_no"));
        orderInfo.setWlnUid(wlnOrder.getString("uid"));
        orderInfo.setWlnCreateTime(wlnOrder.getDate("create_time"));
        orderInfo.setWlnApproveTime(wlnOrder.getDate("approve_time"));
        orderInfo.setModifyTime(wlnOrder.getLong("modify_time"));
        orderInfo.setRemark(wlnOrder.getString("remark"));
        orderInfo.setProvince(wlnOrder.getString("province"));
        orderInfo.setCity(wlnOrder.getString("city"));
        orderInfo.setCounty(wlnOrder.getString("district"));
        orderInfo.setDetailedAddress(wlnOrder.getString("address"));
        orderInfo.setPostcode(wlnOrder.getString("zip"));
        orderInfo.setWlnStatus(wlnOrder.getInteger("process_status"));

        StringBuilder builder = new StringBuilder();
        // 订单来源平台
        if (wlnOrder.containsKey("source_platform")) {
            builder.append(wlnOrder.getString("source_platform"));
        }
        // 店铺名称
        if (wlnOrder.containsKey("shop_name")) {
            builder.append("_");
            builder.append(wlnOrder.getString("shop_name"));
        }
        orderInfo.setCommercePlatform(builder.toString());

        if (department != null) {
            orderInfo.setDepartmentId(department.getId());
            if (Objects.equals(department.getOrderMode(), "2")) {
                orderInfo.setStatus(OrderStatusEnum.REJECT.getKey());
            } else {
                orderInfo.setStatus(OrderStatusEnum.UNDER_REVIEW.getKey());
            }
        } else {
            orderInfo.setStatus(OrderStatusEnum.UNDER_REVIEW.getKey());
        }

        if (orderInfo.getWlnStatus() == 10 || orderInfo.getWlnStatus() == 11 || orderInfo.getWlnStatus() == 12) {
            orderInfo.setStatus(OrderStatusEnum.EXCEPTION.getKey());
        }

        return orderInfo;
    }

    /**
     * 创建订单sku
     */
    private OrderSku createOrderSku(JSONObject wlnOrderSku, OrderInfo orderInfo, SkuSpec skuSpec) {
        OrderSku orderSku = new OrderSku();
        orderSku.setId(IdWorker.getId());
        orderSku.setOrderId(orderInfo.getId());
        orderSku.setQuantity(wlnOrderSku.getBigDecimal("size"));
        orderSku.setPrintType(1);
        orderSku.setWlnSkuId(wlnOrderSku.getString("sys_goods_uid"));
        orderSku.setWlnSkuSpecId(wlnOrderSku.getString("sys_spec_uid"));
        if (skuSpec != null) {
            orderSku.setSkuId(skuSpec.getSkuId());
            orderSku.setSkuSpecId(skuSpec.getId());
        }
        return orderSku;
    }

}
